@@ -1,10 +1,10 @@
 import enum
-import importlib
 import json
 import logging
 import os
 import platform
 import sys
+import textwrap
 import uuid
 
 import click
@@ -12,12 +12,18 @@ import click
 import great_expectations.exceptions as ge_exceptions
 from great_expectations import DataContext, rtd_url_ge_version
 from great_expectations.cli import toolkit
+from great_expectations.cli.cli_messages import NO_DATASOURCES_FOUND
 from great_expectations.cli.docs import build_docs
-from great_expectations.cli.init_messages import NO_DATASOURCES_FOUND
 from great_expectations.cli.mark import Mark as mark
-from great_expectations.cli.util import cli_message, cli_message_dict
+from great_expectations.cli.util import (
+    CLI_ONLY_SQLALCHEMY_ORDERED_DEPENDENCY_MODULE_NAMES,
+    cli_message,
+    cli_message_dict,
+    verify_library_dependent_modules,
+)
 from great_expectations.core import ExpectationSuite
 from great_expectations.core.usage_statistics.usage_statistics import send_usage_message
+from great_expectations.data_context.types.base import DatasourceConfigSchema
 from great_expectations.datasource import (
     PandasDatasource,
     SparkDFDatasource,
@@ -45,12 +51,6 @@ class DatasourceTypes(enum.Enum):
     # TODO DBT = "dbt"
 
 
-DATASOURCE_TYPE_BY_DATASOURCE_CLASS = {
-    "PandasDatasource": DatasourceTypes.PANDAS,
-    "SparkDFDatasource": DatasourceTypes.SPARK,
-    "SqlAlchemyDatasource": DatasourceTypes.SQL,
-}
-
 MANUAL_GENERATOR_CLASSES = ManualBatchKwargsGenerator
 
 
@@ -112,15 +112,17 @@ def delete_datasource(directory, datasource):
     except ValueError:
         cli_message(
             "<red>{}</red>".format(
-                "Datasource {} could not be found".format(datasource)
+                "Datasource {} could not be found.".format(datasource)
             )
         )
         sys.exit(1)
-    else:
+    try:
+        context.get_datasource(datasource)
+    except ValueError:
         cli_message("<green>{}</green>".format("Datasource deleted successfully."))
-
-    if context.get_datasource(datasource) is None:
-        cli_message("<red>{}</red>".format("Datasource not deleted"))
+        sys.exit(1)
+    else:
+        cli_message("<red>{}</red>".format("Datasource not deleted."))
         sys.exit(1)
 
 
@@ -337,6 +339,13 @@ What are you processing your files with?
 def _add_pandas_datasource(
     context, passthrough_generator_only=True, prompt_for_datasource_name=True
 ):
+    send_usage_message(
+        data_context=context,
+        event="cli.new_ds_choice",
+        success=True,
+        event_payload={"type": "pandas"},
+    )
+
     if passthrough_generator_only:
         datasource_name = "files_datasource"
         configuration = PandasDatasource.build_configuration()
@@ -370,49 +379,41 @@ def _add_pandas_datasource(
             }
         )
 
-    context.add_datasource(
-        name=datasource_name, class_name="PandasDatasource", **configuration
+        configuration["class_name"] = "PandasDatasource"
+        configuration["module_name"] = "great_expectations.datasource"
+        errors = DatasourceConfigSchema().validate(configuration)
+        if len(errors) != 0:
+            raise ge_exceptions.GreatExpectationsError(
+                "Invalid Datasource configuration: {:s}".format(errors)
+            )
+
+    cli_message(
+        """
+Great Expectations will now add a new Datasource '{:s}' to your deployment, by adding this entry to your great_expectations.yml:
+
+{:s}
+""".format(
+            datasource_name,
+            textwrap.indent(toolkit.yaml.dump({datasource_name: configuration}), "  "),
+        )
     )
+
+    toolkit.confirm_proceed_or_exit(
+        continuation_message="Okay, exiting now. To learn more about adding datasources, run great_expectations "
+        "datasource --help or visit https://docs.greatexpectations.io/"
+    )
+
+    context.add_datasource(name=datasource_name, **configuration)
     return datasource_name
 
 
-def load_library(library_name, install_instructions_string=None):
-    """
-    Dynamically load a module from strings or raise a helpful error.
-
-    :param library_name: name of the library to load
-    :param install_instructions_string: optional - used when the install instructions
-            are different from 'pip install library_name'
-    :return: True if the library was loaded successfully, False otherwise
-    """
-    try:
-        _ = importlib.import_module(library_name)
-        return True
-    except ModuleNotFoundError:
-        if install_instructions_string:
-            cli_message(
-                """<red>ERROR: Great Expectations relies on the library `{}` to connect to your data.</red>
-            - Please `{}` before trying again.""".format(
-                    library_name, install_instructions_string
-                )
-            )
-        else:
-            cli_message(
-                """<red>ERROR: Great Expectations relies on the library `{}` to connect to your data.</red>
-      - Please `pip install {}` before trying again.""".format(
-                    library_name, library_name
-                )
-            )
-
-        return False
-
-
 def _add_sqlalchemy_datasource(context, prompt_for_datasource_name=True):
+
     msg_success_database = (
         "\n<green>Great Expectations connected to your database!</green>"
     )
 
-    if not load_library("sqlalchemy"):
+    if not _verify_sqlalchemy_dependent_modules():
         return None
 
     db_choices = [str(x) for x in list(range(1, 1 + len(SupportedDatabases)))]
@@ -429,6 +430,13 @@ def _add_sqlalchemy_datasource(context, prompt_for_datasource_name=True):
 
     selected_database = list(SupportedDatabases)[selected_database]
 
+    send_usage_message(
+        data_context=context,
+        event="cli.new_ds_choice",
+        success=True,
+        event_payload={"type": "sqlalchemy", "db": selected_database.name},
+    )
+
     datasource_name = "my_{}_db".format(selected_database.value.lower())
     if selected_database == SupportedDatabases.OTHER:
         datasource_name = "my_database"
@@ -450,31 +458,31 @@ def _add_sqlalchemy_datasource(context, prompt_for_datasource_name=True):
         cli_message(msg_db_config.format(datasource_name))
 
         if selected_database == SupportedDatabases.MYSQL:
-            if not load_library("pymysql"):
+            if not _verify_mysql_dependent_modules():
                 return None
+
             credentials = _collect_mysql_credentials(default_credentials=credentials)
         elif selected_database == SupportedDatabases.POSTGRES:
-            if not load_library("psycopg2"):
+            if not _verify_postgresql_dependent_modules():
                 return None
+
             credentials = _collect_postgres_credentials(default_credentials=credentials)
         elif selected_database == SupportedDatabases.REDSHIFT:
-            if not load_library("psycopg2"):
+            if not _verify_redshift_dependent_modules():
                 return None
+
             credentials = _collect_redshift_credentials(default_credentials=credentials)
         elif selected_database == SupportedDatabases.SNOWFLAKE:
-            if not load_library(
-                "snowflake",
-                install_instructions_string="pip install snowflake-sqlalchemy",
-            ):
+            if not _verify_snowflake_dependent_modules():
                 return None
+
             credentials = _collect_snowflake_credentials(
                 default_credentials=credentials
             )
         elif selected_database == SupportedDatabases.BIGQUERY:
-            if not load_library(
-                "pybigquery", install_instructions_string="pip install pybigquery"
-            ):
+            if not _verify_bigquery_dependent_modules():
                 return None
+
             credentials = _collect_bigquery_credentials(default_credentials=credentials)
         elif selected_database == SupportedDatabases.OTHER:
             sqlalchemy_url = click.prompt(
@@ -495,12 +503,35 @@ def _add_sqlalchemy_datasource(context, prompt_for_datasource_name=True):
             cli_message(
                 "<cyan>Attempting to connect to your database. This may take a moment...</cyan>"
             )
+
             configuration = SqlAlchemyDatasource.build_configuration(
                 credentials="${" + datasource_name + "}"
             )
-            context.add_datasource(
-                name=datasource_name, class_name="SqlAlchemyDatasource", **configuration
+
+            configuration["class_name"] = "SqlAlchemyDatasource"
+            configuration["module_name"] = "great_expectations.datasource"
+            errors = DatasourceConfigSchema().validate(configuration)
+            if len(errors) != 0:
+                raise ge_exceptions.GreatExpectationsError(
+                    "Invalid Datasource configuration: {:s}".format(errors)
+                )
+
+            cli_message(
+                """
+Great Expectations will now add a new Datasource '{0:s}' to your deployment, by adding this entry to your great_expectations.yml:
+
+{1:s}
+The credentials will be saved in uncommitted/config_variables.yml under the key '{0:s}'
+""".format(
+                    datasource_name,
+                    textwrap.indent(
+                        toolkit.yaml.dump({datasource_name: configuration}), "  "
+                    ),
+                )
             )
+
+            toolkit.confirm_proceed_or_exit()
+            context.add_datasource(name=datasource_name, **configuration)
             cli_message(msg_success_database)
             break
         except ModuleNotFoundError as de:
@@ -521,9 +552,9 @@ def _add_sqlalchemy_datasource(context, prompt_for_datasource_name=True):
                 # TODO this message about continuing may not be accurate
                 cli_message(
                     """
-We saved datasource {0:s} in {1:s} and the credentials you entered in {2:s}.
+We saved datasource {:s} in {:s} and the credentials you entered in {:s}.
 Since we could not connect to the database, you can complete troubleshooting in the configuration files documented here:
-<blue>https://docs.greatexpectations.io/en/latest/tutorials/add-sqlalchemy-datasource.html?utm_source=cli&utm_medium=init&utm_campaign={3:s}#{4:s}</blue> .
+<blue>https://docs.greatexpectations.io/en/latest/tutorials/add-sqlalchemy-datasource.html?utm_source=cli&utm_medium=init&utm_campaign={:s}#{:s}</blue> .
 
 After you connect to the datasource, run great_expectations init to continue.
 
@@ -554,7 +585,7 @@ def _collect_postgres_credentials(default_credentials=None):
     if default_credentials is None:
         default_credentials = {}
 
-    credentials = {"drivername": "postgres"}
+    credentials = {"drivername": "postgresql"}
 
     credentials["host"] = click.prompt(
         "What is the host for the postgres connection?",
@@ -588,28 +619,29 @@ def _collect_postgres_credentials(default_credentials=None):
 def _collect_snowflake_credentials(default_credentials=None):
     if default_credentials is None:
         default_credentials = {}
-
     credentials = {"drivername": "snowflake"}
 
-    # required
+    auth_method = click.prompt(
+        """What authentication method would you like to use?
+    1. User and Password
+    2. Single sign-on (SSO)
+    3. Key pair authentication
+""",
+        type=click.Choice(["1", "2", "3"]),
+        show_choices=False,
+    )
 
     credentials["username"] = click.prompt(
         "What is the user login name for the snowflake connection?",
         default=default_credentials.get("username", ""),
     ).strip()
-    credentials["password"] = click.prompt(
-        "What is the password for the snowflake connection?",
-        default="",
-        show_default=False,
-        hide_input=True,
-    )
+
     credentials["host"] = click.prompt(
         "What is the account name for the snowflake connection (include region -- ex "
         "'ABCD.us-east-1')?",
         default=default_credentials.get("host", ""),
     ).strip()
 
-    # optional
     database = click.prompt(
         "What is database name for the snowflake connection? (optional -- leave blank for none)",
         default=default_credentials.get("database", ""),
@@ -642,6 +674,54 @@ def _collect_snowflake_credentials(default_credentials=None):
     if len(role) > 0:
         credentials["query"]["role"] = role
 
+    if auth_method == "1":
+        credentials = {**credentials, **_collect_snowflake_credentials_user_password()}
+    elif auth_method == "2":
+        credentials = {**credentials, **_collect_snowflake_credentials_sso()}
+    elif auth_method == "3":
+        credentials = {**credentials, **_collect_snowflake_credentials_key_pair()}
+    return credentials
+
+
+def _collect_snowflake_credentials_user_password():
+    credentials = {}
+
+    credentials["password"] = click.prompt(
+        "What is the password for the snowflake connection?",
+        default="",
+        show_default=False,
+        hide_input=True,
+    )
+
+    return credentials
+
+
+def _collect_snowflake_credentials_sso():
+    credentials = {}
+
+    credentials["connect_args"] = {}
+    credentials["connect_args"]["authenticator"] = click.prompt(
+        "Valid okta URL or 'externalbrowser' used to connect through SSO",
+        default="externalbrowser",
+        show_default=False,
+    )
+
+    return credentials
+
+
+def _collect_snowflake_credentials_key_pair():
+    credentials = {}
+
+    credentials["private_key_path"] = click.prompt(
+        "Path to the private key used for authentication", show_default=False,
+    )
+
+    credentials["private_key_passphrase"] = click.prompt(
+        "Passphrase for the private key used for authentication (optional -- leave blank for none)",
+        default="",
+        show_default=False,
+    )
+
     return credentials
 
 
@@ -744,7 +824,14 @@ def _collect_redshift_credentials(default_credentials=None):
 def _add_spark_datasource(
     context, passthrough_generator_only=True, prompt_for_datasource_name=True
 ):
-    if not load_library("pyspark"):
+    send_usage_message(
+        data_context=context,
+        event="cli.new_ds_choice",
+        success=True,
+        event_payload={"type": "spark"},
+    )
+
+    if not _verify_pyspark_dependent_modules():
         return None
 
     if passthrough_generator_only:
@@ -785,10 +872,27 @@ def _add_spark_datasource(
                 }
             }
         )
+        configuration["class_name"] = "SparkDFDatasource"
+        configuration["module_name"] = "great_expectations.datasource"
+        errors = DatasourceConfigSchema().validate(configuration)
+        if len(errors) != 0:
+            raise ge_exceptions.GreatExpectationsError(
+                "Invalid Datasource configuration: {:s}".format(errors)
+            )
 
-    context.add_datasource(
-        name=datasource_name, class_name="SparkDFDatasource", **configuration
+    cli_message(
+        """
+Great Expectations will now add a new Datasource '{:s}' to your deployment, by adding this entry to your great_expectations.yml:
+
+{:s}
+""".format(
+            datasource_name,
+            textwrap.indent(toolkit.yaml.dump({datasource_name: configuration}), "  "),
+        )
     )
+    toolkit.confirm_proceed_or_exit()
+
+    context.add_datasource(name=datasource_name, **configuration)
     return datasource_name
 
 
@@ -838,14 +942,14 @@ def get_batch_kwargs(
     context,
     datasource_name=None,
     batch_kwargs_generator_name=None,
-    generator_asset=None,
+    data_asset_name=None,
     additional_batch_kwargs=None,
 ):
     """
     This method manages the interaction with user necessary to obtain batch_kwargs for a batch of a data asset.
 
-    In order to get batch_kwargs this method needs datasource_name, batch_kwargs_generator_name and generator_asset
-    to combine them into a fully qualified data asset identifier(datasource_name/batch_kwargs_generator_name/generator_asset).
+    In order to get batch_kwargs this method needs datasource_name, batch_kwargs_generator_name and data_asset_name
+    to combine them into a fully qualified data asset identifier(datasource_name/batch_kwargs_generator_name/data_asset_name).
     All three arguments are optional. If they are present, the method uses their values. Otherwise, the method
     prompts user to enter them interactively. Since it is possible for any of these three components to be
     passed to this method as empty values and to get their values after interacting with user, this method
@@ -864,9 +968,9 @@ def get_batch_kwargs(
     :param context:
     :param datasource_name:
     :param batch_kwargs_generator_name:
-    :param generator_asset:
+    :param data_asset_name:
     :param additional_batch_kwargs:
-    :return: a tuple: (datasource_name, batch_kwargs_generator_name, generator_asset, batch_kwargs). The components
+    :return: a tuple: (datasource_name, batch_kwargs_generator_name, data_asset_name, batch_kwargs). The components
                 of the tuple were passed into the methods as optional arguments, but their values might
                 have changed after this method's execution. If the returned batch_kwargs is None, it means
                 that the batch_kwargs_generator will know to yield batch_kwargs when called.
@@ -889,12 +993,12 @@ def get_batch_kwargs(
             available_data_assets_dict=available_data_assets_dict,
         )
 
-    # if the user provided us with the batch kwargs generator name and the generator asset, we have everything we need -
+    # if the user provided us with the batch kwargs generator name and the data asset, we have everything we need -
     # let's ask the generator to build batch kwargs for this asset - we are done.
-    if batch_kwargs_generator_name is not None and generator_asset is not None:
-        generator = datasource.get_batch_kwargs_generator(batch_kwargs_generator_name)
+    if batch_kwargs_generator_name is not None and data_asset_name is not None:
+        generator = data_source.get_batch_kwargs_generator(batch_kwargs_generator_name)
         batch_kwargs = generator.build_batch_kwargs(
-            generator_asset, **additional_batch_kwargs
+            data_asset_name, **additional_batch_kwargs
         )
         return batch_kwargs
 
@@ -902,7 +1006,7 @@ def get_batch_kwargs(
         context.get_datasource(datasource_name), (PandasDatasource, SparkDFDatasource)
     ):
         (
-            generator_asset,
+            data_asset_name,
             batch_kwargs,
         ) = _get_batch_kwargs_from_generator_or_from_file_path(
             context,
@@ -911,17 +1015,18 @@ def get_batch_kwargs(
         )
 
     elif isinstance(context.get_datasource(datasource_name), SqlAlchemyDatasource):
-        generator_asset, batch_kwargs = _get_batch_kwargs_for_sqlalchemy_datasource(
+        data_asset_name, batch_kwargs = _get_batch_kwargs_for_sqlalchemy_datasource(
             context, datasource_name, additional_batch_kwargs=additional_batch_kwargs
         )
+
     else:
         raise ge_exceptions.DataContextError(
-            "Datasource {0:s} is expected to be a PandasDatasource or SparkDFDatasource, but is {1:s}".format(
+            "Datasource {:s} is expected to be a PandasDatasource or SparkDFDatasource, but is {:s}".format(
                 datasource_name, str(type(context.get_datasource(datasource_name)))
             )
         )
 
-    return datasource_name, batch_kwargs_generator_name, generator_asset, batch_kwargs
+    return (datasource_name, batch_kwargs_generator_name, data_asset_name, batch_kwargs)
 
 
 def _get_batch_kwargs_from_generator_or_from_file_path(
@@ -963,7 +1068,7 @@ We could not determine the format of the file. What is it?
         "4": "json",
     }
 
-    generator_asset = None
+    data_asset_name = None
 
     datasource = context.get_datasource(datasource_name)
     if batch_kwargs_generator_name is not None:
@@ -1001,24 +1106,24 @@ We could not determine the format of the file. What is it?
                 )
             )
 
-            generator_asset_selection = click.prompt(prompt, show_default=False)
+            data_asset_name_selection = click.prompt(prompt, show_default=False)
 
-            generator_asset_selection = generator_asset_selection.strip()
+            data_asset_name_selection = data_asset_name_selection.strip()
             try:
-                data_asset_index = int(generator_asset_selection) - 1
+                data_asset_index = int(data_asset_name_selection) - 1
                 try:
-                    generator_asset = [name[0] for name in available_data_asset_names][
+                    data_asset_name = [name[0] for name in available_data_asset_names][
                         data_asset_index
                     ]
                 except IndexError:
                     pass
             except ValueError:
-                generator_asset = generator_asset_selection
+                data_asset_name = data_asset_name_selection
 
             batch_kwargs = generator.build_batch_kwargs(
-                generator_asset, **additional_batch_kwargs
+                data_asset_name, **additional_batch_kwargs
             )
-            return generator_asset, batch_kwargs
+            return (data_asset_name, batch_kwargs)
 
     # No generator name was passed or the user chose to enter a file path
 
@@ -1099,7 +1204,19 @@ We have saved your setup progress. When you are ready, run great_expectations in
                     )
                     sys.exit(1)
 
-    return generator_asset, batch_kwargs
+    if data_asset_name is None and batch_kwargs.get("path"):
+        try:
+            # Try guessing a filename
+            filename = os.path.split(batch_kwargs.get("path"))[1]
+            # Take all but the last part after the period
+            filename = ".".join(filename.split(".")[:-1])
+            data_asset_name = filename
+        except (OSError, IndexError):
+            pass
+
+    batch_kwargs["data_asset_name"] = data_asset_name
+
+    return (data_asset_name, batch_kwargs)
 
 
 def _get_batch_kwargs_for_sqlalchemy_datasource(
@@ -1120,7 +1237,7 @@ Enter an SQL query
     if additional_batch_kwargs is None:
         additional_batch_kwargs = {}
 
-    generator_asset = None
+    data_asset_name = None
 
     datasource = context.get_datasource(datasource_name)
 
@@ -1187,7 +1304,7 @@ Enter an SQL query
                 try:
                     data_asset_index = int(selection) - 1
                     try:
-                        generator_asset = [
+                        data_asset_name = [
                             name[0] for name in available_data_asset_names
                         ][data_asset_index]
                     except IndexError:
@@ -1200,7 +1317,7 @@ Enter an SQL query
 
             if query is None:
                 batch_kwargs = temp_generator.build_batch_kwargs(
-                    generator_asset, **additional_batch_kwargs
+                    data_asset_name, **additional_batch_kwargs
                 )
                 batch_kwargs.update(temp_table_kwargs)
             else:
@@ -1217,7 +1334,71 @@ Enter an SQL query
         except KeyError as error:
             cli_message("""<red>ERROR: {}</red>""".format(str(error)))
 
-    return generator_asset, batch_kwargs
+    batch_kwargs["data_asset_name"] = data_asset_name
+
+    return data_asset_name, batch_kwargs
+
+
+def _verify_sqlalchemy_dependent_modules() -> bool:
+    return verify_library_dependent_modules(
+        python_import_name="sqlalchemy", pip_library_name="sqlalchemy"
+    )
+
+
+def _verify_mysql_dependent_modules() -> bool:
+    return verify_library_dependent_modules(
+        python_import_name="pymysql",
+        pip_library_name="pymysql",
+        module_names_to_reload=CLI_ONLY_SQLALCHEMY_ORDERED_DEPENDENCY_MODULE_NAMES,
+    )
+
+
+def _verify_postgresql_dependent_modules() -> bool:
+    psycopg2_success: bool = verify_library_dependent_modules(
+        python_import_name="psycopg2",
+        pip_library_name="psycopg2-binary",
+        module_names_to_reload=CLI_ONLY_SQLALCHEMY_ORDERED_DEPENDENCY_MODULE_NAMES,
+    )
+    # noinspection SpellCheckingInspection
+    postgresql_psycopg2_success: bool = verify_library_dependent_modules(
+        python_import_name="sqlalchemy.dialects.postgresql.psycopg2",
+        pip_library_name="psycopg2-binary",
+        module_names_to_reload=CLI_ONLY_SQLALCHEMY_ORDERED_DEPENDENCY_MODULE_NAMES,
+    )
+    return psycopg2_success and postgresql_psycopg2_success
+
+
+def _verify_redshift_dependent_modules() -> bool:
+    # noinspection SpellCheckingInspection
+    postgresql_success: bool = _verify_postgresql_dependent_modules()
+    redshift_success: bool = verify_library_dependent_modules(
+        python_import_name="sqlalchemy_redshift.dialect",
+        pip_library_name="sqlalchemy-redshift",
+        module_names_to_reload=CLI_ONLY_SQLALCHEMY_ORDERED_DEPENDENCY_MODULE_NAMES,
+    )
+    return redshift_success or postgresql_success
+
+
+def _verify_snowflake_dependent_modules() -> bool:
+    return verify_library_dependent_modules(
+        python_import_name="snowflake.sqlalchemy.snowdialect",
+        pip_library_name="snowflake-sqlalchemy",
+        module_names_to_reload=CLI_ONLY_SQLALCHEMY_ORDERED_DEPENDENCY_MODULE_NAMES,
+    )
+
+
+def _verify_bigquery_dependent_modules() -> bool:
+    return verify_library_dependent_modules(
+        python_import_name="pybigquery.sqlalchemy_bigquery",
+        pip_library_name="pybigquery",
+        module_names_to_reload=CLI_ONLY_SQLALCHEMY_ORDERED_DEPENDENCY_MODULE_NAMES,
+    )
+
+
+def _verify_pyspark_dependent_modules() -> bool:
+    return verify_library_dependent_modules(
+        python_import_name="pyspark", pip_library_name="pyspark"
+    )
 
 
 def profile_datasource(
@@ -1421,7 +1602,7 @@ Enter the path (relative or absolute) of the root directory where the data files
 """
 
 msg_prompt_datasource_name = """
-Give your new data source a short name.
+Give your new Datasource a short name.
 """
 
 msg_db_config = """
